@@ -11,6 +11,7 @@ use reqwest::{
     header::{self, HeaderMap, HeaderValue},
     IntoUrl, Proxy, Result as ReqwestResult,
 };
+use resources::project::ForceDeleteProject;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
 use std::{cell::Cell, fmt::Display, path::Path};
@@ -45,6 +46,7 @@ use crate::resources::{
     trigger::{
         AdvanceRequest as TriggerAdvanceRequest, FetchRequest as TriggerFetchRequest,
         GetResponse as GetTriggersResponse, ResetRequest as TriggerResetRequest,
+        TagExceptionsRequest as TagTriggerExceptionsRequest,
     },
     user::{
         CreateRequest as CreateUserRequest, CreateResponse as CreateUserResponse,
@@ -64,10 +66,11 @@ pub use crate::{
             Identifier as BucketIdentifier, Name as BucketName, NewBucket, TransformTag,
         },
         comment::{
-            AnnotatedComment, Comment, CommentFilter, CommentsIterPage, Continuation, Entity,
-            Id as CommentId, Label, Message, MessageBody, MessageSignature, MessageSubject,
-            NewAnnotatedComment, NewComment, NewEntities, NewLabelling, PropertyMap, PropertyValue,
-            Sentiment, SyncCommentsResponse, Uid as CommentUid,
+            AnnotatedComment, Comment, CommentFilter, CommentsIterPage, Continuation,
+            EitherLabelling, Entity, HasAnnotations, Id as CommentId, Label, Message, MessageBody,
+            MessageSignature, MessageSubject, NewAnnotatedComment, NewComment, NewEntities,
+            NewLabelling, PropertyMap, PropertyValue, Sentiment, SyncCommentsResponse,
+            Uid as CommentUid,
         },
         dataset::{
             Dataset, FullName as DatasetFullName, Id as DatasetId, Identifier as DatasetIdentifier,
@@ -85,16 +88,16 @@ pub use crate::{
         project::{NewProject, Project, ProjectName, UpdateProject},
         source::{
             FullName as SourceFullName, Id as SourceId, Identifier as SourceIdentifier,
-            Name as SourceName, NewSource, Source, UpdateSource,
+            Name as SourceName, NewSource, Source, SourceKind, UpdateSource,
         },
         statistics::Statistics,
         trigger::{
             Batch as TriggerBatch, FullName as TriggerFullName, SequenceId as TriggerSequenceId,
-            Trigger,
+            Trigger, TriggerException, TriggerExceptionMetadata,
         },
         user::{
             Email, GlobalPermission, Id as UserId, Identifier as UserIdentifier,
-            ModifiedPermissions, NewUser, Organisation, OrganisationPermission, User, Username,
+            ModifiedPermissions, NewUser, ProjectPermission, User, Username,
         },
     },
 };
@@ -619,6 +622,18 @@ impl Client {
         Ok(())
     }
 
+    pub fn tag_trigger_exceptions(
+        &self,
+        trigger_name: &TriggerFullName,
+        exceptions: &[TriggerException],
+    ) -> Result<()> {
+        self.put::<_, _, serde::de::IgnoredAny>(
+            self.endpoints.trigger_exceptions(trigger_name)?,
+            TagTriggerExceptionsRequest { exceptions },
+        )?;
+        Ok(())
+    }
+
     /// Gets a project.
     pub fn get_project(&self, project_name: &ProjectName) -> Result<Project> {
         let response =
@@ -666,8 +681,18 @@ impl Client {
     }
 
     /// Deletes an existing project.
-    pub fn delete_project(&self, project_name: &ProjectName) -> Result<()> {
-        self.delete(self.endpoints.project_by_name(project_name)?)?;
+    pub fn delete_project(
+        &self,
+        project_name: &ProjectName,
+        force_delete: ForceDeleteProject,
+    ) -> Result<()> {
+        let endpoint = self.endpoints.project_by_name(project_name)?;
+        match force_delete {
+            ForceDeleteProject::No => self.delete(endpoint)?,
+            ForceDeleteProject::Yes => {
+                self.delete_query(endpoint, Some(&json!({ "force": true })))?
+            }
+        };
         Ok(())
     }
 
@@ -1072,6 +1097,18 @@ impl Endpoints {
             })
     }
 
+    fn trigger_exceptions(&self, trigger_name: &TriggerFullName) -> Result<Url> {
+        self.base
+            .join(&format!(
+                "/api/v1/datasets/{}/triggers/{}/exceptions",
+                trigger_name.dataset.0, trigger_name.trigger.0
+            ))
+            .map_err(|source| Error::UrlParseError {
+                source,
+                message: "Could not build URL for trigger exceptions.".to_owned(),
+            })
+    }
+
     fn recent_comments(&self, dataset_name: &DatasetFullName) -> Result<Url> {
         self.base
             .join(&format!("/api/_private/datasets/{}/recent", dataset_name.0))
