@@ -1,5 +1,6 @@
 use chrono::{DateTime, Utc};
 use serde::{Deserialize, Serialize};
+use serde_with::{DeserializeFromStr, SerializeDisplay};
 use std::{
     fmt::{Display, Formatter, Result as FmtResult},
     str::FromStr,
@@ -22,6 +23,9 @@ pub struct Source {
     pub should_translate: bool,
     pub created_at: DateTime<Utc>,
     pub updated_at: DateTime<Utc>,
+
+    #[serde(rename = "_kind")]
+    pub kind: SourceKind,
 }
 
 impl Source {
@@ -104,6 +108,39 @@ impl Display for Identifier {
     }
 }
 
+#[derive(Debug, Clone, SerializeDisplay, DeserializeFromStr, PartialEq, Eq, Hash)]
+pub enum SourceKind {
+    Call,
+    Chat,
+    Unknown(Box<str>),
+}
+
+impl FromStr for SourceKind {
+    type Err = Error;
+
+    fn from_str(string: &str) -> Result<Self> {
+        Ok(match string {
+            "call" => SourceKind::Call,
+            "chat" => SourceKind::Chat,
+            value => SourceKind::Unknown(value.into()),
+        })
+    }
+}
+
+impl Display for SourceKind {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self {
+                SourceKind::Call => "call",
+                SourceKind::Chat => "chat",
+                SourceKind::Unknown(value) => value.as_ref(),
+            }
+        )
+    }
+}
+
 #[derive(Debug, Clone, Serialize, PartialEq, Eq, Default)]
 pub struct NewSource<'request> {
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -123,6 +160,9 @@ pub struct NewSource<'request> {
 
     #[serde(skip_serializing_if = "Option::is_none")]
     pub sensitive_properties: Option<Vec<&'request str>>,
+
+    #[serde(skip_serializing_if = "Option::is_none", rename = "_kind")]
+    pub kind: Option<&'request SourceKind>,
 }
 
 #[derive(Debug, Clone, Serialize, PartialEq, Eq, Default)]
@@ -172,3 +212,34 @@ pub(crate) struct UpdateRequest<'request> {
 pub(crate) struct UpdateResponse {
     pub source: Source,
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn source_kind_roundtrips() {
+        assert_eq!(SourceKind::Call, SourceKind::from_str("call").unwrap());
+        assert_eq!(
+            &serde_json::ser::to_string(&SourceKind::Call).unwrap(),
+            "\"call\""
+        );
+
+        assert_eq!(SourceKind::Chat, SourceKind::from_str("chat").unwrap());
+        assert_eq!(
+            &serde_json::ser::to_string(&SourceKind::Chat).unwrap(),
+            "\"chat\""
+        );
+    }
+
+    #[test]
+    fn unknown_source_kind_roundtrips() {
+        let kind = SourceKind::from_str("unknown").unwrap();
+        match &kind {
+            SourceKind::Unknown(error) => assert_eq!(&**error, "unknown"),
+            _ => panic!("Expected error to be parsed as Unknown(..)"),
+        }
+
+        assert_eq!(&serde_json::ser::to_string(&kind).unwrap(), "\"unknown\"")
+    }
+}
